;; Generated package description from xelb.el  -*- no-byte-compile: t -*-
(define-package "xelb" "0.22" "X protocol Emacs Lisp Binding" '((emacs "27.1") (compat "30")) :commit "9e007c9eb595cd5d58e749c7679201bda999a229" :authors '(("Chris Feng" . "chris.w.feng@gmail.com")) :maintainer '(("Adrián Medraño Calvo" . "adrian@medranocalvo.com") ("Steven Allen" . "steven@stebalien.com") ("Daniel Mendler" . "mail@daniel-mendler.de")) :keywords '("unix") :url "https://github.com/emacs-exwm/xelb")
